"use client";

import { useRouter } from "next/navigation";
import * as Types from "../../types";
import "highlight.js/styles/github-dark.css";
import { deleteArticle, publishArticle, unpublishArticle } from "../../action";
import { useState } from "react";

export default function PostDisplay({
  post,
  loggedIn,
}: {
  post: Types.Post;
  loggedIn: boolean;
}) {
  const router = useRouter();

  const [confirmDelete, setConfirmDelete] = useState(false);

  return (
    <>
      <div
        style={{
          display: "flex",
          flexDirection: "column",
          borderColor: "#999",
          transition: "border-color 0.3s linear",
          borderStyle: "solid",
          marginTop: "1rem",
        }}
      >
        <div
          style={{
            width: "100%",
            backgroundColor: "#999",
            color: "#111",
            display: "flex",
            alignItems: "center",
            padding: "0.5rem",
            boxSizing: "border-box",
            fontSize: "1rem",
          }}
        >
          <span>{post.title}</span>
        </div>
        <div
          style={{
            display: "flex",
            flexDirection: "row",
            justifyContent: "space-between",
            fontSize: "0.9rem",
            maxWidth: "100%",
            flexWrap: "wrap",
          }}
        >
          <div
            style={{
              display: "flex",
              justifyContent: "flex-start",
              padding: "0.5rem",
              gap: 4,
            }}
          >
            tags: {post.tags.join(", ")}
          </div>
          <div
            style={{
              display: "flex",
              justifyContent: "flex-end",
              padding: "0.5rem",
              gap: 4,
            }}
          >
            {post.is_draft && (
              <>
                <span>draft - </span>
                <div
                  style={{
                    cursor: "pointer",
                    color: "#999",
                    transition: "color 0.3s linear",
                  }}
                  onMouseOver={(e) => {
                    e.currentTarget.style.color = "#eee";
                  }}
                  onMouseLeave={(e) => {
                    e.currentTarget.style.color = "#999";
                  }}
                  onClick={async () => {
                    await publishArticle(post.slug);
                    router.refresh();
                  }}
                >
                  move to published
                </div>
              </>
            )}
            {!post.is_draft && (
              <>
                <div>
                  <div>
                    <span>published: </span>
                    <span>{post.publishedDate.toLocaleDateString()}</span>
                  </div>
                  {loggedIn && (
                    <div
                      style={{
                        cursor: "pointer",
                        color: "#999",
                        transition: "color 0.3s linear",
                      }}
                      onMouseOver={(e) => {
                        e.currentTarget.style.color = "#eee";
                      }}
                      onMouseLeave={(e) => {
                        e.currentTarget.style.color = "#999";
                      }}
                      onClick={async () => {
                        await unpublishArticle(post.slug);
                        router.refresh();
                      }}
                    >
                      move to drafts
                    </div>
                  )}
                </div>
              </>
            )}
          </div>
        </div>
        <div
          style={{
            width: "100%",
            height: "2px",
            backgroundColor: "#999",
          }}
        ></div>
        <div
          style={{
            display: "flex",
            flexDirection: "column",
            padding: "1rem 0.5rem 0 0.5rem",
          }}
        >
          <div style={{ fontSize: "0.9rem" }}>
            <div dangerouslySetInnerHTML={post.contentRendered} />
          </div>
        </div>
      </div>
      <div
        style={{
          display: "flex",
          justifyContent: "space-between",
          marginTop: "1rem",
        }}
      >
        {loggedIn ? (
          <div style={{ display: "flex", gap: "1rem" }}>
            <div
              style={{
                backgroundColor: "#6a6",
                color: "#111",
                fontSize: "0.8rem",
                padding: "0.5rem",
                transition: "background-color 0.3s linear",
                userSelect: "none",
                cursor: "pointer",
              }}
              onMouseOver={(e) => {
                e.currentTarget.style.backgroundColor = "#8c8";
              }}
              onMouseOut={(e) => {
                e.currentTarget.style.backgroundColor = "#6a6";
              }}
              onClick={() => router.push(`/blog/write/${post.slug}`)}
            >
              edit
            </div>
            <div
              style={{
                backgroundColor: "#a66",
                color: "#111",
                fontSize: "0.8rem",
                padding: "0.5rem",
                transition: "background-color 0.3s linear",
                userSelect: "none",
                cursor: "pointer",
              }}
              onMouseOver={(e) => {
                e.currentTarget.style.backgroundColor = "#c88";
              }}
              onMouseOut={(e) => {
                e.currentTarget.style.backgroundColor = "#a66";
              }}
              onClick={() => setConfirmDelete(true)}
            >
              delete
            </div>
            <DeleteModal
              open={confirmDelete}
              onCancel={() => setConfirmDelete(false)}
              onConfirm={async () => {
                await deleteArticle(post.slug);
                router.push("/blog");
              }}
            />
          </div>
        ) : (
          <div></div>
        )}
        <span
          style={{
            backgroundColor: "#999",
            color: "#111",
            fontSize: "0.8rem",
            padding: "0.5rem",
            transition: "background-color 0.3s linear",
            userSelect: "none",
            cursor: "pointer",
          }}
          onMouseOver={(e) => {
            e.currentTarget.style.backgroundColor = "#eee";
          }}
          onMouseOut={(e) => {
            e.currentTarget.style.backgroundColor = "#999";
          }}
          onClick={() => router.push("/blog")}
        >
          return to all posts
        </span>
      </div>
    </>
  );
}

function DeleteModal({
  open,
  onCancel,
  onConfirm,
}: {
  open: boolean;
  onCancel: () => void;
  onConfirm: () => void;
}) {
  return (
    <div
      id="deleteModal"
      style={{
        display: open
          ? "flex"
          : "none" /* Will be changed to "flex" when opened */,
        position: "fixed",
        top: 0,
        left: 0,
        width: "100%",
        height: "100%",
        backgroundColor: "rgba(0, 0, 0, 0.7)",
        zIndex: 1000,
        justifyContent: "center",
        alignItems: "center",
      }}
      onClick={(e) => {
        if (e.target == e.currentTarget) {
          onCancel();
        }
      }}
    >
      <div
        style={{
          backgroundColor: "#222",
          padding: "1.5rem",
          borderRadius: "4px",
          maxWidth: "400px",
          width: "90%",
          boxShadow: "0 0 10px rgba(0, 0, 0, 0.5)",
        }}
        onClick={(e) => e.stopPropagation()}
      >
        <h3 style={{ margin: "0 0 1rem 0", color: "#eee" }}>
          Confirm Deletion
        </h3>
        <p style={{ marginBottom: "1.5rem", color: "#ccc" }}>
          Are you sure you want to delete this post? This action cannot be
          undone.
        </p>
        <div
          style={{
            display: "flex",
            justifyContent: "flex-end",
            gap: "0.75rem",
          }}
        >
          <button
            style={{
              backgroundColor: "#a66",
              color: "#fff",
              border: "none",
              padding: "0.5rem 1rem",
              cursor: "pointer",
              borderRadius: "2px",
            }}
            onClick={() => {
              onConfirm();
            }}
          >
            Yes, delete it
          </button>
          <button
            style={{
              backgroundColor: "#555",
              color: "#fff",
              border: "none",
              padding: "0.5rem 1rem",
              cursor: "pointer",
              borderRadius: "2px",
            }}
            onClick={() => {
              onCancel();
            }}
          >
            Cancel
          </button>
        </div>
      </div>
    </div>
  );
}
